(function() {
  // ====== [UTILS] ======

  function codeGen(length, special) {
    var iteration = 0
    var password = ''
    var randomNumber
    if (special === undefined) {
      special = false
    }
    while (iteration < length) {
      randomNumber = (Math.floor((Math.random() * 100)) % 94) + 33
      if (!special) {
        if ((randomNumber >= 33) && (randomNumber <= 47)) { continue }
        if ((randomNumber >= 58) && (randomNumber <= 64)) { continue }
        if ((randomNumber >= 91) && (randomNumber <= 96)) { continue }
        if ((randomNumber >= 123) && (randomNumber <= 126)) { continue }
      }
      iteration++
      password += String.fromCharCode(randomNumber)
    }
    return password
  }

  window.code_gen = codeGen

  //* **************************************************************************

  function fmdbExecute(sql, args, success, error) {
    window.nativeDatabase.execute(
      sql, args,
      success || _.noop,
      error || _.noop
    )
  }

  function fmdbQueryValues(sql, args, success, error) {
    window.nativeDatabase.queryValues(
      sql, args,
      function(res) {

        // Rezygnujemy z .map() na rzecz zwykłego for
        // z powodu dziwnego błędu, prawdopodobnie
        // gdzieś na mostku z kodem natywnym, który
        // powodował, że:
        // res = ["trial"]
        // res.map(JSON.parse) => []
        // var ret = []
        // for (var i = 0; i < res.length; i++) {
        //   ret.push(JSON.parse(res[i]))
        // } => [trial]
        //
        // success(res.map(JSON.parse))
        var ret = []
        for (var i = 0, length = res.length; i < length; i++) {
          ret.push(JSON.parse(res[i]))
        }
        success(ret)
      },
      error || _.noop
    )
  }

  //* **************************************************************************

  // ====== [ WebSQLStore ] ======

  window.FMDBStore = function(tableName, success, error) {
    this.tableName = tableName

    fmdbExecute(
      'CREATE TABLE IF NOT EXISTS `' + tableName + '` (id unique, value)',
      [], success, error
    )
  }

  Backbone.LocalStorage = window.Store = window.FMDBStore

  window.db = null
  window.WebSQLStore = function(db, name) {
    return new Backbone.LocalStorage(name)
  }

  _.extend(window.FMDBStore.prototype, {
    create: function(model, success, error) {
      if (!model.id) {
        model.id = model.attributes.id = codeGen(16)
        model.attributes.is_new = true
      }

      fmdbExecute(
        'INSERT OR REPLACE INTO `' + this.tableName + '` VALUES (?,?)',
        [model.id, JSON.stringify(model.toJSON())], success, error
      )
    },

    destroy: function(model, success, error) {
      fmdbExecute(
        'DELETE FROM `' + this.tableName + '` WHERE id=?',
        [model.id], success, error
      )
    },

    find: function(model, success, error) {
      fmdbQueryValues(
        'SELECT id, value FROM `' + this.tableName + '` WHERE id=?',
        [model.id], success, error
      )
    },

    findAll: function(model, success, error) {
      fmdbQueryValues(
        'SELECT id, value FROM `' + this.tableName + '`',
        [], success, error
      )
    },

    update: function(model, success, error) {
      fmdbExecute(
        'INSERT OR REPLACE INTO `' + this.tableName + '` VALUES (?,?)',
        [model.id, JSON.stringify(model.toJSON())], success, error
      )
    },

    destroyAll: function(success) {
      fmdbExecute('DELETE FROM `' + this.tableName + '`', [], success)
    },
  })

  // ====== [ Backbone.sync WebSQL implementation ] ======

  Backbone.sync = function(method, model, options) {
    var store = model.store || model.collection.store

    if (store === null) {
      console.error('[BACKBONE-WEBSQL] model without store object -> ', model)
      return
    }

    function action(methodName) {
      switch (methodName) {
        case 'read': return model.id ? store.find : store.findAll
        case 'create': return store.create
        case 'update': return store.update
        case 'delete': return store.destroy
        default:
          console.error(methodName)
          return _.noop
      }
    }

    method = action(method).bind(store)
    method(model, options.success, options.error)
  }

  // LocalStorage shim for Mac
  // eslint-disable-next-line
  var appConfig = new window.FMDBStore('app_config',
    function() {
      var nozLocalStorage = {}

      nozLocalStorage.setItem = function(key, value) {
        fmdbExecute(
          'INSERT OR REPLACE INTO `' + appConfig.tableName + '` VALUES (?,?)',
          [key, value]
        )
      }

      nozLocalStorage.getItem = function(key) {
        var result

        fmdbQueryValues(
          'SELECT id, value FROM `' + appConfig.tableName + '` WHERE id=?',
          [key],
          function(res) {
            result = res[0]
          }
        )

        return result
      }

      nozLocalStorage.clear = function() {
        fmdbExecute('DELETE FROM `' + appConfig.tableName + '`', [])
      }

      window.nozLocalStorage = nozLocalStorage
    },
    function() {
      console.error('app config error')
    }
  )
})()
